     /*
 DSPRobotics
 PIC Clicker LED Dispaly - HID  V1.00    (PIC Clicker)
 Language: Mikro C Pro
 Chip: P18F47J53
 CLK: 48Mhz
 
  HID Data:
 Char 0 Read:
 bit 0 = LED1
 bit 1 = LED2

 Char 0 Write:
 bit 0 = Button1
 bit 1 = Button2
 Char 1
 Temp

 Aplication: Ideal for using with the FlowStone Graphical Programming Language.
 www.dsprobotics.com

 */
/*---------------------------------------------------------------*/
// USB buffers
unsigned char readbuff[64] absolute 0x500;   // Buffers should be in USB RAM, please consult datasheet
unsigned char writebuff[64] absolute 0x540;
//General
char kk;
char i;
// Globals
float Temp;
// pin definitions
//LEDs
sbit LD1 at LATA0_bit;    //LED 1
sbit LD2 at LATA1_bit;    //LED 2
sbit LD1_Direction at TRISA0_bit;
sbit LD2_Direction at TRISA1_bit;
//Button Inputs
sbit T1 at RD3_bit;   //Button 1
sbit T2 at RD2_bit;   //Button 2
sbit T1_Direction at TRISD3_bit;
sbit T2_Direction at TRISD2_bit;
// Ir Termo click constants
const _IR_THERMO_ADDR = 0x5A;
const _AMB_TEMP      = 0x06;
const _OBJ_TEMP      = 0x07;
char txt[10];
/*---------------------------------------------------------------*/

// Interrupt routine
void interrupt() {

 USB_Interrupt_Proc();                   // USB servicing is done inside the interrupt
}
/*---------------------------------------------------------------*/
void clear_USB_buffers()
{
   for (i=0; i<63;i++)
   {
   readbuff[i] = 0;
   writebuff[i] = 0;
   }
}
/*---------------------------------------------------------------*/
char read_button1() {
  if (Button(&PORTD, 3, 2, 0)) {               // Detect logical one
      return  1;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
char read_button2() {
  if (Button(&PORTD, 2, 2, 0)) {               // Detect logical one
      return  2;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
/*****************************************************************
* Function ReadSensor(char Temp_Source)
* Overview: Function Reads Ir Termo sensor via I2C interface
* Input: Temp_Source  0x06 for Ambient temperature
*                     0x07 for Target temperature
* Output: Value that's proportional to Temperature in Kelvin degrees
*******************************************************************/
float ReadSensor(char Temp_Source){
  unsigned int Temp_var;

  I2C2_Start();                  // issue I2C start signal
  I2C2_Wr(_IR_THERMO_ADDR << 1); // send byte via I2C  (device address << 1)
  I2C2_Wr(Temp_Source);          // send byte (data address)
  I2C2_Repeated_Start();         // issue I2C signal repeated start
  I2C2_Wr(_IR_THERMO_ADDR << 1); // send byte (device address << 1)
  Temp_var = I2C2_Rd(0);         // Read the data (NO acknowledge)
  Temp_var = (I2C2_Rd(0) << 8) + Temp_var;
  I2C2_Stop();                   // issue I2C stop signal

  return Temp_var;
}
/*---------------------------------------------------------------*/
void load_AMB() {
//Writes Temp text to USB buffer
 for (i =0;i <10;i++) {
    writebuff[1+i]= txt[i];
 }
}
/*---------------------------------------------------------------*/
void load_OBJ() {
//Writes Temp text to USB buffer
 for (i =0;i <10;i++) {
    writebuff[11+i]= txt[i];
 }
}
/*---------------------------------------------------------------*/
void main(){
  long cnt = 0;

  ANCON0 = 0xFF;               // Default all pins to digital
  ANCON1 = 0x1F;               //
  TRISD = 0;
  TRISB = 0;
  TRISA = 0;
  RCSTA1 = 1;

  //Buttons
  T1_Direction = 1;         // Set direction for buttons
  T2_Direction = 1;
  //Leds
  LD1_Direction = 0;        // Set direction for LEDS
  LD2_Direction = 0;
  LD1 = 0;
  LD2 = 0;                 // turn off leds
  //I2C
  I2C2_Init(50000);                     // performs I2C initialization
  Delay_ms(1000);

 //HID
  clear_USB_buffers();
  HID_Enable(&readbuff,&writebuff);       // Enable HID communication
 /*---------------------------------------------------------------*/
 //Main Loop
  while (1){
  //USB
  kk = HID_Read();
  if (kk != 0)
  {
   //IrTemp
   Temp = ReadSensor(_AMB_TEMP);          // Read Ambient temperature
   Temp = (Temp * 0.02) - 273.15;         // Convert result in Celsius degrees
   FloatToStr(Temp, txt);
   load_AMB();                            //Loads Temp into HID Buffer
   
   Temp = ReadSensor(_OBJ_TEMP);          // Read Ambient temperature
   Temp = (Temp * 0.02) - 273.15;         // Convert result in Celsius degrees
   FloatToStr(Temp, txt);
   load_OBJ();                            //Loads Temp into HID Buffer
   //LEDS
   if ((readbuff[0] & 0x01 )> 0) {
     LD1 = 1;
   }  else LD1 = 0;
   if ((readbuff[0] & 0x02 )> 0) {
     LD2 = 1;
   }  else LD2 = 0;
   //Buttons
   writebuff[0]=0;
   writebuff[0] = writebuff[0] | read_button1() ;
   writebuff[0] = writebuff[0] | read_button2() ;
   //HID
   HID_Write(&writebuff,64);
  }
 }
}